//
//  SRField.h
//  StorageRoomRestKit
//
//  Created by Sascha Konietzke on 7/13/11.
//  Copyright 2011 Thriventures. All rights reserved.
//

#import "SREmbedded.h"

/**
 * The embedded Field Resource
 *
 * @see SRCollection
 */
@interface SRField : SREmbedded {
    NSString *name;
    NSString *identifier;
    NSString *hint;
    NSString *inputType;
    NSNumber *required;
    NSNumber *unique;
    NSNumber *maximumLength;
    NSNumber *minimumLength;
    NSNumber *maximumSize;
    NSNumber *minimumSize;
    NSNumber *maximumNumber;
    NSNumber *minimumNumber;
}

/**
 * The name of the Field.
 */
@property (nonatomic, copy) NSString *name;

/**
 * The identifier of the Field that is used in the API.
 */
@property (nonatomic, copy) NSString *identifier;

/**
 * The hint displayed in the editing interface.
 */
@property (nonatomic, copy) NSString *hint;

/**
 * The input type used in the editing interface.
 */
@property (nonatomic, copy) NSString *inputType;

/**
 * Required validation rule.
 */
@property (nonatomic, retain) NSNumber *required;

/**
 * Uniqueness validation rule.
 */
@property (nonatomic, retain) NSNumber *unique;


@property (nonatomic, retain) NSNumber *maximumLength;
@property (nonatomic, retain) NSNumber *minimumLength;
@property (nonatomic, retain) NSNumber *maximumSize;
@property (nonatomic, retain) NSNumber *minimumSize;
@property (nonatomic, retain) NSNumber *maximumNumber;
@property (nonatomic, retain) NSNumber *minimumNumber;

/**
 * Adds the Field to an object mapping.
 */
- (void)addToObjectMapping:(RKObjectMapping *)anObjectMapping;

/**
 * Adds the Field to an auto-generated Entry Class.
 */
- (BOOL)addToEntryClass:(Class)aClass;

/**
 * Release the instance variables that have been added by addToEntryClass
 */
- (void)releaseFromAutoGeneratedEntry:(id)anEntry;

@end
